# -*- coding: utf-8 -*-
from distutils.core import setup

setup(
    name="xunitgen",
    version="1.0.1",
    description="Generate xUnit.xml files",
    author="Nicolas Léveillé",
    author_email="nicolas@uucidl.com",
    url="https://github.com/uucidl/uu.xunitgen",
    packages=["xunitgen"],
    license='MIT',
    long_description=open('README.rst').read(),
)
